//
//  TwitterApi.m
//  DevExample
//
//  Created by Martin Jacala on 11/24/10.
//  Copyright 2010 __MyCompanyName__. All rights reserved.
//

#import "TwitterApi.h"


@implementation TwitterApi
@synthesize xmlParser, model, characters, apiConnection, stringData, modelDelegate;

static NSString* kApiEndpoint = @"http://api.twitter.com/1/statuses/public_timeline.xml";

- (id)initWithService:(NSString *)service delegate:(id)delegate {
	[self setModelDelegate:delegate];
// obtain data from twitter url:
	[self setApiConnection:[NSURLConnection connectionWithRequest:[NSURLRequest requestWithURL:[NSURL URLWithString:kApiEndpoint]] 
														 delegate:self]];
	 return self;
}


- (void)dealloc {
	self.xmlParser = nil;
	self.model = nil;
	self.characters = nil;
	self.modelDelegate = nil;
	self.stringData = nil;
	self.apiConnection = nil;
    [super dealloc];
}

#pragma mark -
#pragma mark URL Connection callbacks

- (void)connection:(NSURLConnection *)connection didReceiveResponse:(NSURLResponse *)response {
	[self setStringData:[NSMutableData data]];
}

- (void)connection:(NSURLConnection *)connection didReceiveData:(NSData *)data {
	[self.stringData appendData:data];
}

- (void)connectionDidFinishLoading:(NSURLConnection *)connection {
// parse downloaded data
	NSXMLParser *parser = [[NSXMLParser alloc] initWithData:self.stringData];
	self.stringData = nil;
	
	[self setXmlParser:parser];
	[parser release];

	[self.xmlParser setDelegate:self];
	[self.xmlParser parse];	
}

- (void)connection:(NSURLConnection *)connection didFailWithError:(NSError *)error {
	NSLog(@"Error: %@", error);
}


#pragma mark -
#pragma mark XML Processing


- (void)parserDidStartDocument:(NSXMLParser *)parser {
	if (!self.model) {
		[self setModel:[NSMutableArray array]];
	}
}


- (void)parserDidEndDocument:(NSXMLParser *)parser {
	// TODO: create protocol
	[self.modelDelegate apiCallFinished:self withData:self.model];
}

- (void)parser:(NSXMLParser *)parser didStartElement:(NSString *)elementName 
										namespaceURI:(NSString *)namespaceURI 
									   qualifiedName:(NSString *)qualifiedName 
										  attributes:(NSDictionary *)attributeDict {
	if ([elementName isEqualToString:@"status"]) {
		[self.model addObject:[NSMutableDictionary dictionary]];
	} 
}

- (void)parser:(NSXMLParser *)parser didEndElement:(NSString *)elementName 
									  namespaceURI:(NSString *)namespaceURI 
									qualifiedName:(NSString *)qName {
	
	if ([elementName isEqualToString:@"created_at"]  ||
		[elementName isEqualToString:@"screen_name"] || 
		[elementName isEqualToString:@"text"] ||
		[elementName isEqualToString:@"location"] || 
		[elementName hasSuffix:@"point"] ||
		[elementName isEqualToString:@"profile_image_url"]) {
		[(NSMutableDictionary*)[self.model lastObject] setObject:[[self.characters stringByReplacingOccurrencesOfString:@"\n  " withString:@""] stringByReplacingOccurrencesOfString:@" " withString:@"" options:NSCaseInsensitiveSearch range:NSMakeRange(0, 2)] forKey:elementName];	
	}
	
	
	self.characters = nil; 
}

- (void)parser:(NSXMLParser *)parser foundCharacters:(NSString *)string {
	if (self.characters) {
		[self.characters appendString:string];
	} else {
		[self setCharacters:[NSMutableString stringWithString:string]];
	}
}

@end
